/* Deep Waters v0.3.0 */
define('deep-waters',['exports'],function(exports){'use strict';/**
 * Takes a series of validator functions and return a new functions that evaluate whether the given value
 * is a valid value for all the previous validators.
 * It also removes duplicated validators
 * @param validators
 * @returns {function(*=): boolean}
 */
var compose = function compose() {
  for (var _len = arguments.length, validators = new Array(_len), _key = 0; _key < _len; _key++) {
    validators[_key] = arguments[_key];
  }

  return function (value) {
    return Array.from(new Set(validators)).every(function (validator) {
      return validator(value);
    });
  };
};/**
 * Takes a value and return true if it's of type number
 * @param value
 * @returns {boolean}
 */
var isNumber = function isNumber(value) {
  return typeof value === 'number';
};var biggerThan = function biggerThan(number) {
  return compose(isNumber, function (value) {
    return value > number;
  });
};var lessThan = function lessThan(number) {
  return compose(isNumber, function (value) {
    return value < number;
  });
};var between = function between(min) {
  return function (max) {
    return compose(biggerThan(min), lessThan(max));
  };
};/**
 * Takes a value and return true if it's of type string
 * @param value
 * @returns {boolean}
 */
var isString = function isString(value) {
  return typeof value === 'string';
};/**
 * Check whether the given string contains the defined prop
 * @param prop
 * @returns {function(*=): boolean}
 */

var contains = function contains(prop) {
  return compose(isString, function (value) {
    return value.indexOf("".concat(prop)) >= 0;
  });
};/**
 * Returns true if the given value is of type function
 *
 * According to: https://stackoverflow.com/questions/5999998/check-if-a-variable-is-of-function-type
 * @param fn
 * @returns {boolean|boolean}
 */
var isFunction = function isFunction(fn) {
  return !!fn && {}.toString.call(fn) === '[object Function]';
};/**
 * Makes sure that every item of the array is valid for the received validator
 * @param validator
 * @returns {function(*): *}
 */

var every = function every(validator) {
  return compose(function (value) {
    return isFunction(validator) ? value : false;
  }, Array.isArray, function (items) {
    return items.every(validator);
  });
};/**
 * Takes a function and return a new function which will return the opposite of the given one
 * @param fn
 * @returns {function(*=): boolean}
 */
var not = function not(fn) {
  return function (value) {
    return !fn(value);
  };
};/**
 * Takes a series of validator functions and return a new functions that evaluate whether the given value
 * is a valid value for at least one of the previous validators.
 * @param validators
 * @returns {function(*=): boolean}
 */
var or = function or() {
  for (var _len = arguments.length, validators = new Array(_len), _key = 0; _key < _len; _key++) {
    validators[_key] = arguments[_key];
  }

  return function (value) {
    return validators.reduce(function (acc, current) {
      return acc || current(value);
    }, false);
  };
};/**
 * Takes a value and returns true only if it is of type boolean and of value `false`
 * @param value
 * @returns {boolean}
 */
var isFalse = function isFalse(value) {
  return value === false;
};/**
 * Returns true if the given value is zero
 * @param value
 * @returns {boolean}
 */
var isZero = function isZero(value) {
  return value === 0;
};/**
 * Checks if a given variable is null
 * @param value
 * @returns {boolean}
 */
var isNull = function isNull(value) {
  return value === null;
};/**
 * Takes a value and returns true if it's undefined
 * @param value
 * @returns {boolean}
 */
var isUndefined = function isUndefined(value) {
  return typeof value === 'undefined';
};/**
 * Takes a value and returns true only if it's of type `string` and it's empty
 * @type {function(*=): boolean}
 */

var isEmptyString = compose(isString, function (str) {
  return str.trim() === '';
});/**
 * Returns true if the given value is falsy (false, zero, empty string, null, undefined or NaN)
 * @type {function(*=): *|boolean}
 */

var isFalsy = or(isFalse, isZero, isEmptyString, isNull, isUndefined, function (value) {
  return !value;
});/**
 * Takes a value and returns true if it's not falsy
 * @param value
 * @returns {boolean}
 */

var isTruthy = not(isFalsy);/**
 * Takes a prop and returns a new function evaluating that the received object has the defined prop.
 * @param prop
 * @returns {function(*=): boolean}
 */

var hasProp = function hasProp(prop) {
  return compose(isTruthy, function (value) {
    return prop && value[prop] !== undefined;
  });
};/**
 * Returns a validator for the defined shape
 * @param shape
 * @returns {function(*): boolean}
 */
var hasShape = function hasShape(shape) {
  return function (object) {
    return Object.keys(shape).every(function (prop) {
      var validator = shape[prop];
      var result = false;

      try {
        result = validator(object[prop]);
      } catch (e) {
        return false;
      }

      return result;
    });
  };
};/**
 * returns true if the given value is a valid boolean
 * @param value
 * @returns {boolean}
 */
var isBoolean = function isBoolean(value) {
  return value === true || value === false;
};var isDivisibleBy = function isDivisibleBy(number) {
  return compose(isNumber, function (value) {
    return value % number === 0;
  });
};var matchesPattern = function matchesPattern(regex) {
  return function (str) {
    return !!regex.test(str);
  };
};var isEmail = compose(isString, matchesPattern(/^\w+([.-]?\w+)*@\w+([.-]?\w+)*(\.\w{2,3})+$/));var when = function when(condition, fn) {
  return function (value) {
    if (condition(value)) return fn(value);
    return true;
  };
};/**
 * Takes number and returns a function that returns true if the length property of its first parameter
 * (may it be an array or a string) equals the given number.
 * @param number
 * @returns {function({length: *}): boolean}
 */

var ofLength = function ofLength(number) {
  return compose(hasProp('length'), function (_ref) {
    var length = _ref.length;
    return length === number;
  });
};function _typeof(obj) {
  "@babel/helpers - typeof";

  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}/**
 * Honestly, I'm not going to comment this function
 * @returns {boolean}
 */
var stubFalse = function stubFalse() {
  return false;
};/**
 * Returns true if (and only) the given value is a valid object
 * @type {function(*=): boolean}
 */

var isObject = compose(when(isNull, stubFalse), function (value) {
  return _typeof(value) === 'object' && value.constructor === Object;
});/**
 * Returns true if the given value is empty (can be a string, an array or an object)
 * @type {function(*=): boolean}
 */

var isEmpty = compose(when(Array.isArray, ofLength(0)), when(isString, isEmptyString), when(isObject, function (value) {
  return isEmpty(Object.keys(value));
}));var ipv4Regex = /^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/;
/**
 * Takes a value and return true if it's a valid IP v4 string
 * @param value
 * @returns {boolean}
 */

var isIPv4 = compose(isString, matchesPattern(ipv4Regex));var ipv6Regex = /(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))/;
/**
 * Takes a value and return true if it's a valid IP v6 string
 * @param value
 * @returns {boolean}
 */

var isIPv6 = compose(isString, matchesPattern(ipv6Regex));/**
 * Takes a value and return true if it's a valid IP (v4 or v6) string
 * @param value
 * @returns {boolean}
 */

var isIP = or(isIPv4, isIPv6);/* eslint-disable max-len */

var isInternational = matchesPattern(/\+(9[976]\d|8[987530]\d|6[987]\d|5[90]\d|42\d|3[875]\d|2[98654321]\d|9[8543210]|8[6421]|6[6543210]|5[87654321]|4[987654310]|3[9643210]|2[70]|7|1)\d{1,14}$/);
var isUK = matchesPattern(/^(?:(?:\(?(?:0(?:0|11)\)?[\s-]?\(?|\+)44\)?[\s-]?(?:\(?0\)?[\s-]?)?)|(?:\(?0))(?:(?:\d{5}\)?[\s-]?\d{4,5})|(?:\d{4}\)?[\s-]?(?:\d{5}|\d{3}[\s-]?\d{3}))|(?:\d{3}\)?[\s-]?\d{3}[\s-]?\d{3,4})|(?:\d{2}\)?[\s-]?\d{4}[\s-]?\d{4}))(?:[\s-]?(?:x|ext\.?|#)\d{3,4})?$/);
var isUSA = matchesPattern(/1?\W*([2-9][0-8][0-9])\W*([2-9][0-9]{2})\W*([0-9]{4})(\se?x?t?(\d*))?/);
var isGermany = matchesPattern(/^([+][0-9]{1,3}[ .-])?([(]{1}[0-9]{1,6}[)])?([0-9 .\-/]{3,20})((x|ext|extension)[ ]?[0-9]{1,4})?$/);
var isChina = matchesPattern(/1[34578][012356789]\d{8}|134[012345678]\d{7}/);
var isRussia = matchesPattern(/^((\+7|7|8)+([0-9]){10})$/);
var isJapan = matchesPattern(/^(?:\d{10}|\d{3}-\d{3}-\d{4}|\d{2}-\d{4}-\d{4}|\d{3}-\d{4}-\d{4})$/);
/* eslint-enable max-len */

var isPhone = compose(isString, or(isInternational, isUSA, isUK, isGermany, isUSA, isChina, isRussia, isJapan));/**
 * Takes a value and returns true if it exists;
 * It's a semantic variation of `isTruthy`
 * @param value
 * @returns {boolean}
 */

var isRequired = isTruthy;/**
 * Takes a value and returns true only if it is of type boolean and of value `true`
 * @param value
 * @returns {boolean}
 */
var isTrue = function isTrue(value) {
  return value === true;
};/**
 * Returns true if the given string matches a valid url, according to:
 * https://stackoverflow.com/questions/5717093/check-if-a-javascript-string-is-a-url
 * @type {function(*=): boolean}
 */

var isURL = compose(not(isEmptyString), matchesPattern(new RegExp('^(https?:\\/\\/)?' // protocol
+ '((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|' // domain name
+ '((\\d{1,3}\\.){3}\\d{1,3}))' // OR ip (v4) address
+ '(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*' // port and path
+ '(\\?[;&a-z\\d%_.~+=-]*)?' // query string
+ '(\\#[-a-z\\d_]*)?$', 'i')));var maxLength = function maxLength(max) {
  return compose(hasProp('length'), function (_ref) {
    var length = _ref.length;
    return length <= max;
  });
};var minLength = function minLength(min) {
  return compose(hasProp('length'), function (_ref) {
    var length = _ref.length;
    return length >= min;
  });
};exports.between=between;exports.biggerThan=biggerThan;exports.compose=compose;exports.contains=contains;exports.every=every;exports.hasProp=hasProp;exports.hasShape=hasShape;exports.isBoolean=isBoolean;exports.isDivisibleBy=isDivisibleBy;exports.isEmail=isEmail;exports.isEmpty=isEmpty;exports.isEmptyString=isEmptyString;exports.isFalse=isFalse;exports.isFalsy=isFalsy;exports.isFunction=isFunction;exports.isIP=isIP;exports.isIPv4=isIPv4;exports.isIPv6=isIPv6;exports.isNull=isNull;exports.isNumber=isNumber;exports.isObject=isObject;exports.isPhone=isPhone;exports.isRequired=isRequired;exports.isString=isString;exports.isTrue=isTrue;exports.isTruthy=isTruthy;exports.isURL=isURL;exports.isUndefined=isUndefined;exports.isZero=isZero;exports.lessThan=lessThan;exports.matchesPattern=matchesPattern;exports.maxLength=maxLength;exports.minLength=minLength;exports.not=not;exports.ofLength=ofLength;exports.or=or;exports.when=when;Object.defineProperty(exports,'__esModule',{value:true});});//# sourceMappingURL=deep-waters.amd.js.map
